//
// for OpenPTC 1.0 C++ Implementation
//

// include files
#include "ptc.h"
#include <math.h>
#include <fstream.h>

int STARTX=0;
int STARTY=0;

int WIDTH=800;
int HEIGHT=600;

#define PI 3.141592653589793238462643383279502884197


#ifdef __DEBUG__

    //
    // This checking is performed only when __DEBUG__ is defined,
#else

    // In release build no checking is performed for the sake of efficiency.

#endif



//Example of inline assembly 16-bit opcodes
/*
unsigned int XMSalloc(long size)
{
      int rval = (int) (size / 1024L);

      if(size % 1024L)
            rval++;

      if(!initFlag)
            return 0;

      _asm
      {
            mov dx, rval;
            mov ax, 0x0900;
      }

      if(XMSDriver())
            _asm mov rval, dx;
      else  rval = 0;

      return rval;
}
*/



void load(Surface &surface,char filename[])
{
    // open image file
    ifstream is(filename,ios::in|ios::binary|ios::nocreate);

    // check image file stream
    if (!is) throw Error("could not load image");

    // skip header
    is.seekg(18);

    // get surface dimensions
    const int width  = surface.width();
    const int height = surface.height();
    
    // allocate image pixels
    char8 *pixels = new char8[width*height*3];

    // read image pixels one line at a time
    for (int y=height-1; y>=0; y--) is.read((char*)&pixels[width*y*3],width*3);

    // load pixels to surface
    surface.load(pixels,width,height,width*3,Format(24,0x00FF0000,0x0000FF00,0x000000FF),Palette());

    // free image pixels
    delete[] pixels;
}


void save(Surface &surface,char filename[])
{
    // open image file for writing
    ofstream os(filename,ios::out|ios::binary);

    // check image file stream
    if (!os) throw Error("could not open image file for writing");

    // get surface dimensions
    const int width  = surface.width();
    const int height = surface.height();
    
    // generate the header for a true color targa image
    char header[18] = { 0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0 };

    // set targa image width
    header[12] = width & 0xFF;
    header[13] = width >> 8;
    
    // set targa image height
    header[14] = height & 0xFF;
    header[15] = height >> 8;

    // set bits per pixel
    header[16] = 24;

    // write tga header
    os.write(header,18);

    // calculate size of image pixels
    const int size = width * height * 3;
    
    // allocate image pixels
    char8 *pixels = new char8[size];

    // save surface to image pixels
    surface.save(pixels,width,height,width*3,Format(24,0x00FF0000,0x0000FF00,0x000000FF),Palette());

    // write image pixels one line at a time
    for (int y=height-1; y>=0; y--) os.write((char*)&pixels[width*y*3],width*3);

    // free image pixels
    delete[] pixels;
}


inline int calculate(float real,float imaginary,int maximum)
{
    // complex number 'c'
    const float c_r = real;
    const float c_i = imaginary;

    // complex 'z'
    float z_r = 0;
    float z_i = 0;
    
    // complex 'z' squares
    float z_r_squared = 0;
    float z_i_squared = 0;

    // mandelbrot function iteration loop
    for (int count=0; count<maximum; count++)
    {
        // square 'z' and add 'c'
        z_i = 2 * z_r * z_i + c_i;
        z_r = z_r_squared - z_i_squared + c_r;

        // update 'z' squares
        z_r_squared = z_r * z_r;
        z_i_squared = z_i * z_i;

        // calculate squared magnitude of complex 'z'
        const float z_squared_magnitude = z_r_squared + z_i_squared;

        // stop iterating if the magnitude of 'z' is greater than two
        if (z_squared_magnitude>=4) return count;
    }

    // maximum
    return 0;
}


void mandelbrot(Console &console,Surface &surface,float x1,float y1,float x2,float y2)
{
    // constant values
    const int entries = 1024;
    const int maximum = 1024;

    // fractal color table
    int32 table[entries];

    // generate fractal color table
    for (int i=0; i<entries; i++)
    {
        // useful pi constant
        const float pi = 3.1415926f;

        // calculate normalized index
        const float index = i / (float) entries;

        // calculate sine curve time value
        const float time = index * pi - pi/2;

        // lookup sine curve intensity at time and scale to [0,1]
        float intensity = (float) ( sin(time) + 1.0f ) / 2.0f;

        // raise the intensity to a power
        intensity = (float) pow(intensity,0.1);

        // store intensity as a shade of blue
        table[i] = (int32) ( 255 * intensity );
    }

    // lock surface pixels
    int32 *pixels = (int32*) surface.lock();

    // get surface dimensions
    const int width  = surface.width();
    const int height = surface.height();

    // current pixel pointer
    int32 *pixel = pixels;

    // calculate real x,y deltas
    const float dx = (x2 - x1) / width;
    const float dy = (y2 - y1) / height;

    // imaginary axis
    float imaginary = y1;

    // iterate down surface y
    for (int y=0; y<height; y++)
    {
        // real axis
        float real = x1;

        // iterate across surface x
        for (int x=0; x<width; x++)
        {
            // calculate the mandelbrot interation count
            const int count = calculate(real,imaginary,maximum);

            // calculate color table index
            const int index = count % entries;

            // lookup color from iteration
            const int32 color = table[index];

            // store color
            *pixel = color;

            // next pixel
            pixel ++;

            // update real
            real += dx;
        }

        // update imaginary
        imaginary += dy;

        // setup line area
        Area area(0,y,width,y+1);

        // copy surface area to console
        surface.copy(console,area,area);

        // update console area
        console.update(area);
    }

    // unlock surface
    surface.unlock();
}





void blit_not_rotated(Surface &surface,
				  float x,float y,
				  Surface &dotsurface,
				  float rotation,
				  float colour_r,
				  float colour_g,
				  float colour_b)
{
	if ((colour_r != 1.0f) || (colour_g != 1.0f) || (colour_b != 1.0f))
	{
		// lock surface pixels
		unsigned char *pixels = (unsigned char*) surface.lock();
		unsigned char *dotpixels = (unsigned char*) dotsurface.lock();

		// get surface dimensions
		const int width  = surface.width();
		const int height = surface.height();
		// get surface dimensions
		const int dotwidth  = dotsurface.width();
		const int dotheight = dotsurface.height();

		// current pixel pointer
		unsigned char *pixel = pixels;
		unsigned char *dotpixel = dotpixels;

		int xx,yy;

		for (yy=0; yy<dotheight; yy++)
		{
			for (xx=0; xx<dotwidth; xx++)
			{
				float dotred = float(dotpixel[yy*dotwidth*4 + xx*4 + 0])/256.0f;
				float dotgreen = float(dotpixel[yy*dotwidth*4 + xx*4 + 1])/256.0f;
				float dotblue = float(dotpixel[yy*dotwidth*4 + xx*4 + 2])/256.0f;


				pixel[int(y+yy)*width*4 + int(x+xx)*4 + 0] = unsigned char(float(pixel[int(y+yy)*width*4 + int(x+xx)*4 + 0])/256.0f * float(1-((1-dotred) * (1 - colour_r)))*256.0f);
				pixel[int(y+yy)*width*4 + int(x+xx)*4 + 1] = unsigned char(float(pixel[int(y+yy)*width*4 + int(x+xx)*4 + 1])/256.0f * float(1-((1-dotgreen) * (1 - colour_g)))*256.0f);
				pixel[int(y+yy)*width*4 + int(x+xx)*4 + 2] = unsigned char(float(pixel[int(y+yy)*width*4 + int(x+xx)*4 + 2])/256.0f * float(1-((1-dotblue) * (1 - colour_b)))*256.0f);
			}
		}

		// unlock surface
		surface.unlock();
		dotsurface.unlock();
	}
}






void blit_rotated_not_optimized(Surface &surface,
				  double x,double y,
				  Surface &dotsurface,
				  double rotation,
				  double scale,
				  double colour_r,
				  double colour_g,
				  double colour_b)
{
	if ((colour_r != 1.0f) || (colour_g != 1.0f) || (colour_b != 1.0f))
	{
		// lock surface pixels
		unsigned char *pixels = (unsigned char*) surface.lock();
		unsigned char *dotpixels = (unsigned char*) dotsurface.lock();

		// get surface dimensions
		const int width  = surface.width();
		const int height = surface.height();
		// get surface dimensions
		const int dotwidth  = dotsurface.width();
		const int dotheight = dotsurface.height();

		// current pixel pointer
		unsigned char *pixel = pixels;
		unsigned char *dotpixel = dotpixels;

		double x2,y2, dotx, doty;
		double dotred, dotgreen, dotblue;
		int x2_start, x2_end;
		int y2_start, y2_end;

// Determine the maximum size of a rotated dot - to know
// how large area we have to blit into the surface
// Depends on scale and rotation
		double blitsize=((double)dotwidth/2*scale*sqrt(2));
		if (dotheight>dotwidth)
		{
			blitsize = ((double)dotheight/2*scale*sqrt(2));
		}
		
//Calculate boundaries for this blit
		y2_start=(int)(y-blitsize);
		y2_end=(int)(y+blitsize);
		if (y2_start < 0)
		{
			y2_start = 0;
		}
		if (y2_end >= height-1)
		{
			y2_end = height-1;
		}
//The same for x-coordinates
		x2_start=(int)(x-blitsize);
		x2_end=(int)(x+blitsize);
		if (x2_start < 0)
		{
			x2_start = 0;
		}
		if (x2_end >= width-1)
		{
			x2_end = width-1;
		}

		double dotx_frac;
		double doty_frac;
		double dot_brightness;

		if ((y2_start <= height-1) && (y2_end >= 0) &&
			(x2_start <= width-1) && (x2_end >= 0))
		{

			for (y2=y2_start; y2<=y2_end; y2++)
			{
				for (x2=x2_start; x2<=x2_end; x2++)
				{

//Matlab code for rotation				
//dot_x = dot_centerx + (cos(rotation) * xx - sin(rotation) * yy)/dot_scale;
//dot_y = dot_centery + (sin(rotation) * xx + cos(rotation) * yy)/dot_scale;

// Get the position of where this xx,yy will point to in
// the dotpicture
					dotx = (double)dotwidth/2 + ((cos(rotation) * (x2 - x)) - (sin(rotation) * (y2 - y))) / scale;
					doty = (double)dotheight/2 + ((sin(rotation) * (x2 - x)) + (cos(rotation) * (y2 - y))) / scale;
// Only plot the pixel if inside the dotpicture
					if ((dotx >= 0) && (dotx < (dotwidth-1)) && (doty >= 0) && (doty < (dotheight-1)))
					{
// Get colourvalue for this pixel in the dotpicture
						dotx_frac=dotx - (int)dotx;
						doty_frac=doty - (int)doty;
//Add together the colourvalues for the four pixels closest to our destination pixel
// All of these are "weighted" against how close to the pixel we are
//
//   a    b
//      X
//
//
//   c    d
//
//If X is the destination-pixel in the dotpicture
// (which is a fractional number, and we only have integer numbered pixels
// present in our picture, we get the following colour-value:

						dotred = (1-dotx_frac) * (1-doty_frac) * double(dotpixel[(int)doty*dotwidth*4 + (int)dotx*4 + 0])/256.0f;
						dotgreen = (1-dotx_frac) * (1-doty_frac) * double(dotpixel[(int)doty*dotwidth*4 + (int)dotx*4 + 1])/256.0f;
						dotblue = (1-dotx_frac) * (1-doty_frac) * double(dotpixel[(int)doty*dotwidth*4 + (int)dotx*4 + 2])/256.0f;

						dotred += (dotx_frac) * (1-doty_frac) * double(dotpixel[(int)doty*dotwidth*4 + (int)(dotx+1)*4 + 0])/256.0f;
						dotgreen += (dotx_frac) * (1-doty_frac) * double(dotpixel[(int)doty*dotwidth*4 + (int)(dotx+1)*4 + 1])/256.0f;
						dotblue += (dotx_frac) * (1-doty_frac) * double(dotpixel[(int)doty*dotwidth*4 + (int)(dotx+1)*4 + 2])/256.0f;

						dotred += (1-dotx_frac) * (doty_frac) * double(dotpixel[(int)(doty+1)*dotwidth*4 + (int)dotx*4 + 0])/256.0f;
						dotgreen += (1-dotx_frac) * (doty_frac) * double(dotpixel[(int)(doty+1)*dotwidth*4 + (int)dotx*4 + 1])/256.0f;
						dotblue += (1-dotx_frac) * (doty_frac) * double(dotpixel[(int)(doty+1)*dotwidth*4 + (int)dotx*4 + 2])/256.0f;

						dotred += (dotx_frac) * (doty_frac) * double(dotpixel[(int)(doty+1)*dotwidth*4 + (int)(dotx+1)*4 + 0])/256.0f;
						dotgreen += (dotx_frac) * (doty_frac) * double(dotpixel[(int)(doty+1)*dotwidth*4 + (int)(dotx+1)*4 + 1])/256.0f;
						dotblue += (dotx_frac) * (doty_frac) * double(dotpixel[(int)(doty+1)*dotwidth*4 + (int)(dotx+1)*4 + 2])/256.0f;

//Calculate "how white is white" for this pixel
// Use this value to normalize the current pixel so that white actually becomes white
// All areas filled with one colour will have an even colour when rotated

						dot_brightness = (1-dotx_frac) * (1-doty_frac) +
										(dotx_frac) * (1-doty_frac) +
										(1-dotx_frac) * (doty_frac) +
										(dotx_frac) * (doty_frac);

// The scaling is done further below
//						dotred = dotred/dot_brightness;
//						dotgreen = dotgreen/dot_brightness;
//						dotblue = dotblue/dot_brightness;
						if ((dotred < 0.99f) || (dotgreen < 0.99f) || (dotblue < 0.99f))
						{
							pixel[int(y2)*width*4 + int(x2)*4 + 0] = unsigned char(double(pixel[int(y2)*width*4 + int(x2)*4 + 0])/256.0f * double(1-((1-dotred/dot_brightness) * (1 - colour_r)))*256.0f);
							pixel[int(y2)*width*4 + int(x2)*4 + 1] = unsigned char(double(pixel[int(y2)*width*4 + int(x2)*4 + 1])/256.0f * double(1-((1-dotgreen/dot_brightness) * (1 - colour_g)))*256.0f);
							pixel[int(y2)*width*4 + int(x2)*4 + 2] = unsigned char(double(pixel[int(y2)*width*4 + int(x2)*4 + 2])/256.0f * double(1-((1-dotblue/dot_brightness) * (1 - colour_b)))*256.0f);
						}
					}
				}
			}
		}
		// unlock surface
		surface.unlock();
		dotsurface.unlock();
	}
}


















void blit_rotated(Surface &surface,
				  double x,double y,
				  Surface &dotsurface,
				  double rotation,
				  double scale,
				  double colour_r,
				  double colour_g,
				  double colour_b)
{
	if ((colour_r != 1.0f) || (colour_g != 1.0f) || (colour_b != 1.0f))
	{
		// lock surface pixels
		unsigned char *pixels = (unsigned char*) surface.lock();
		unsigned char *dotpixels = (unsigned char*) dotsurface.lock();

		// get surface dimensions
		const int width  = surface.width();
		const int height = surface.height();
		// get surface dimensions
		const int dotwidth  = dotsurface.width();
		const int dotheight = dotsurface.height();

		// current pixel pointer
		unsigned char *pixel = pixels;
		unsigned char *dotpixel = dotpixels;

		double x2,y2;
		int dotx, doty;
		unsigned int dotred, dotgreen, dotblue;
		int x2_start, x2_end;
		int y2_start, y2_end;

// Determine the maximum size of a rotated dot - to know
// how large area we have to blit into the surface
// Depends on scale and rotation
		double blitsize=((double)dotwidth/2*scale*sqrt(2));
		if (dotheight>dotwidth)
		{
			blitsize = ((double)dotheight/2*scale*sqrt(2));
		}
		
//Calculate boundaries for this blit
		y2_start=(int)(y-blitsize);
		y2_end=(int)(y+blitsize);
		if (y2_start < 0)
		{
			y2_start = 0;
		}
/*		if (y2_end >= height-1)
		{
			y2_end = height-1;
		}*/
//This is for the scrolling version
		if (y2_end >= height-3)
		{
			y2_end = height-3;
		}
//The same for x-coordinates
		x2_start=(int)(x-blitsize);
		x2_end=(int)(x+blitsize);
		if (x2_start < 0)
		{
			x2_start = 0;
		}
		if (x2_end >= width-1)
		{
			x2_end = width-1;
		}

		int dotx_frac;
		int doty_frac;
		int index, index2;
		int coeff1,coeff2,coeff3,coeff4;
		const threshold = 254 * 256;
		int coeffr,coeffg,coeffb;
		double cosrotation, sinrotation;

		if ((y2_start <= height-1) && (y2_end >= 0) &&
			(x2_start <= width-1) && (x2_end >= 0))
		{

			cosrotation=cos(rotation);
			sinrotation=sin(rotation);

			coeffr=(255 - (int)(colour_r*255.0f));
			coeffg=(255 - (int)(colour_g*255.0f));
			coeffb=(255 - (int)(colour_b*255.0f));

			for (y2=y2_start; y2<=y2_end; y2++)
			{
				index2 = int(y2)*width*4 + int(x2_start)*4 - 4;
				for (x2=x2_start; x2<=x2_end; x2++)
				{
					index2 += 4;
//Matlab code for rotation				
//dot_x = dot_centerx + (cos(rotation) * xx - sin(rotation) * yy)/dot_scale;
//dot_y = dot_centery + (sin(rotation) * xx + cos(rotation) * yy)/dot_scale;

// Get the position of where this xx,yy will point to in
// the dotpicture
					dotx = (int)(((double)dotwidth/2  + ((cosrotation * (x2 - x)) - (sinrotation * (y2 - y))) / scale) * 256.0f);
					doty = (int)(((double)dotheight/2 + ((sinrotation * (x2 - x)) + (cosrotation * (y2 - y))) / scale) * 256.0f);
// Only plot the pixel if inside the dotpicture
					if ((dotx >= 0) && (dotx < ((dotwidth-1) << 8)) && (doty >= 0) && (doty < ((dotheight-1) << 8)))
					{
// Get colourvalue for this pixel in the dotpicture
						dotx_frac=dotx - ((dotx >> 8) << 8);
						doty_frac=doty - ((doty >> 8) << 8);
//						dotx_frac=(int)(255.0f*(dotx - (int)dotx));
//						doty_frac=(int)(255.0f*(doty - (int)doty));
//						dotx_frac=dotx - (int)dotx;
//						doty_frac=doty - (int)doty;
//Add together the colourvalues for the four pixels closest to our destination pixel
// All of these are "weighted" against how close to the pixel we are
//
//   a    b
//      X
//
//
//   c    d
//
//If X is the destination-pixel in the dotpicture
// (which is a fractional number, and we only have integer numbered pixels
// present in our picture, we get the following colour-value:

						index=(doty >> 8)*dotwidth*4 + (dotx >> 8)*4;
						coeff1 = (255-dotx_frac) * (255-doty_frac);
						coeff2 = (dotx_frac) * (255-doty_frac);
						coeff3 = (255-dotx_frac) * (doty_frac);
						coeff4 = (dotx_frac) * (doty_frac);

						dotred = (coeff1 * dotpixel[index]) >> 8;
						dotgreen = (coeff1 * dotpixel[index + 1]) >> 8;
						dotblue = (coeff1 * dotpixel[index + 2]) >> 8;

						dotred += (coeff2 * dotpixel[index + 4]) >> 8;
						dotgreen += (coeff2 * dotpixel[index + 5]) >> 8;
						dotblue += (coeff2 * dotpixel[index + 6]) >> 8;

						dotred += (coeff3 * dotpixel[index + dotwidth*4]) >> 8;
						dotgreen += (coeff3 * dotpixel[index + dotwidth*4 + 1]) >> 8;
						dotblue += (coeff3 * dotpixel[index + dotwidth*4 + 2]) >> 8;

						dotred += (coeff4 * dotpixel[index + dotwidth*4 + 4]) >> 8;
						dotgreen += (coeff4 * dotpixel[index + dotwidth*4 + 5]) >> 8;
						dotblue += (coeff4 * dotpixel[index + dotwidth*4 + 6]) >> 8;

//Use these to check the calculations above
//						dotred = (dotpixel[index]) << 8;
//						dotgreen = (dotpixel[index + 1]) << 8;
//						dotblue = (dotpixel[index + 2]) << 8;


						if ((dotred < threshold) || (dotgreen < threshold) || (dotblue < threshold))
						{
							pixel[index2 + 0] = (unsigned char)((unsigned int)(pixel[index2 + 0] * (65535-((255-(dotred   >> 8)) * coeffr))) >> 16);
							pixel[index2 + 1] = (unsigned char)((unsigned int)(pixel[index2 + 1] * (65535-((255-(dotgreen >> 8)) * coeffg))) >> 16);
							pixel[index2 + 2] = (unsigned char)((unsigned int)(pixel[index2 + 2] * (65535-((255-(dotblue  >> 8)) * coeffb))) >> 16);
						}
					}
				}
			}
		}
		// unlock surface
		surface.unlock();
		dotsurface.unlock();
	}
}





void gubbrora(Console &console,Surface &surface,int startx,int starty, Surface &dotsurface)
{
    // get surface dimensions
    const int width  = surface.width();
    const int height = surface.height();
    // get surface dimensions
    const int dotwidth  = dotsurface.width();
    const int dotheight = dotsurface.height();
    // setup copy area
    Area area(0,0,width,height);

    surface.clear(Color(0.0f,0.0f,0.0f));
	surface.copy(console,area,area);
	console.update(area);
/*
	blit_not_rotated(surface, 400, 300, dotsurface, 0.2f, 1.0f, 0.5f, 0.25f);
	surface.copy(console,area,area);
	console.update(area);
	blit_not_rotated(surface, 300, 200, dotsurface, 0.1f, 0.5f, 0.25f, 1.0f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 400, 300, dotsurface, 0.1f,  0.6f, 0.5f, 1.0f, 0.25f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 100, 100, dotsurface, -0.4f, 0.6f, 0.1f, 1.0f, 0.1f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 500, 200, dotsurface, 0.9f,  2.6f, 1.0f, 1.0f, 0.0f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 400, 100, dotsurface, 3.14f, 4.6f, 0.5f, 0.5f, 0.5f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 300, 100, dotsurface, 2.14f, 0.2f, 0.8f, 0.5f, 0.8f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 330, 100, dotsurface, 1.14f, 0.2f, 0.8f, 0.5f, 0.8f);
	surface.copy(console,area,area);
	console.update(area);
	blit_rotated(surface, 360, 100, dotsurface, 1.54f, 0.2f, 0.8f, 0.5f, 0.8f);
*/


	int t=0;
	int x=400;
	int x_dir=7;
	double spin=0;
	double spin_speed=0.1;
	float fadeinr=0.0f;
	float fadeing=0.0f;
	float fadeinb=0.0f;


	while (!console.key())
  {
		if (t < 50)
		{
			fadeinr += 0.02f;
			if (fadeinr > 1.0f)
			{
				fadeinr = 1.0f;
			}
			fadeing += 0.07f;
			if (fadeing > 1.0f)
			{
				fadeing = 1.0f;
			}
			fadeinb += 0.04f;
			if (fadeinb > 1.0f)
			{
				fadeinb = 1.0f;
			}
			
			surface.clear(Color(fadeinr,fadeing,fadeinb));
		}
//		else if (t % 200 == 0)
//		{
//			surface.clear(Color(1.0f,1.0f,1.0f));
//		}
		else
		{
			Area areafrom(0,0,width,height-1);
			Area areato(0,1,width,height);

			surface.copy(surface,areato,areafrom);
		}
		blit_rotated(surface, (double)x, 580.0f - abs((int)((530.0f * sin(0.14f*t) * (0.7f + 0.3f*sin(0.04f*t))))), dotsurface, spin, 0.2f + 0.19f*sin(0.04f*t), 0.5f + 0.5f*sin(0.07f*t), 0.5f + 0.5f*sin(0.05f*t), 0.5f + 0.5f*sin(0.03f*t));
		surface.copy(console,area,area);
		console.update(area);
		t++;
		spin += spin_speed;
		x += x_dir;
		if (x > 760)
		{
			x_dir = -7;
		}
		else if (x < 40)
		{
			x_dir = 7;
		}
	
	}

//    while (!console.key())
//    {
//	    surface.clear(Color(1.0f,1.0f,1.0f));
//		blit_rotated(surface, 800.0f*rand()/RAND_MAX, 600.0f*rand()/RAND_MAX, dotsurface, 2.0f*3.141592653589793238*rand()/RAND_MAX, 0.5f*rand()/RAND_MAX + 0.001f, 0.2f, 0.2f, 0.8f);
//		surface.copy(console,area,area);
//		console.update(area);
//	}


	// copy surface area to console
	// update console area
	surface.copy(console,area,area);
	console.update(area);
}



int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    try
    {
        // create console
        Console console;

        // create format
        Format format(32,0x00FF0000,0x0000FF00,0x000000FF);
 
        // open the console with a single page
        console.open("Save example",WIDTH,HEIGHT,format);

        // create surface matching console dimensions
        Surface surface(console.width(),console.height(),format);

        // setup viewing area
        const float x1 = -2.00f;
        const float y1 = -1.25f;
        const float x2 = +1.00f;
        const float y2 = +1.25f;

		// Get dotpic
//		Format format(32,0x00FF0000,0x0000FF00,0x000000FF);
		Surface dotsurface(236,236,format);
		load(dotsurface,".\\pictures\\dot.tga");

		// render the mandelbrot fractal
//        mandelbrot(console,surface,x1,y1,x2,y2);
        gubbrora(console,surface,STARTX,STARTY,dotsurface);

        // save mandelbrot image
        save(surface,"Save.tga");

        // read key
        console.read();

        // close console
        console.close();
    }
    catch (Error &error)
    {
        // report error
        error.report();
    }

    // exit
    return 0;
}
